// defs.h

// allgemein
#define OWNADR      1  // eigene Adresse

#define NOCRC				0
#define WITHCRC			1

#define INDICATORLAMP		1
#define ACLAMP					2
#define MULTISECTION		4
#define C2							8
#define DIAGNOSTICDATA 16

#define OUT         1
#define IN          2
#define ACK         0x06
#define NAK         0x15
#define TRUE        1
#define FALSE       0
#define MAXMSGLEN   30  // max. Datenlnge nach DLC vom PC
#define MINMSGLEN   3   // min. Datenlnge vom PC
#define PLAIN       1   // alle Variablen resetten
#define UART1MAXRCVLEN 25 // max. Byteanzahl von UART1, die eingesteckt werden knnen
// Framenummern, PC an Simu
#define ADR         0   // ADResse ist das 1. Byte im Array, wird nicht gespeichert
#define DLC         1   // DLC, 2. Byte vom PC, aber das erste, was gespeichert wird -> Index = 0
#define AKT         2   // AKTion ist das 3. Byte vom PC, aber das 2. was gespeichert wird -> index = 1
#define REP_PC      3   // Anzahl Weiderholungen, Parametriesierung v. PC
#define PRIO_PC     4   // Prioritt, Parametrisierung vom PC
#define CYT_PC      5   // Zykluszeit, Parametrisierung vom PC
#define MID_PC      6   // MID, Parametrisierung vom PC
#define PID_PC      7   // PID, Parametrisierung vom PC
#define D1_PC       8   // Date 1, Parametrisierung vom PC
#define D2_PC       9   // Date 2, Parametrisierung vom PC
#define D3_PC       10  // Date 3, Parametrisierung vom PC
#define D4_PC       11  // Date 4, Parametrisierung vom PC

// Framenummern der Messages der J-Bus Kommik
#define REP   0   // Repeat, Anzahl Wiederholungen, REP = 0 immer senden
#define PRIO  1   // Prio-Feld
#define CYT   2   // Cycle-Time
#define PCNT  3   // Prio-Count
#define CCNT  4   // Cycle-Count
#define XLEN  5   // LEN & XMIT-Bit, XMIT = oberstes Bit = Merker, 1 = ob Msg. gefunkt werden soll
#define MID   6   // Message ID (Sender)
#define PID   7
#define D1    8
#define D2    9
#define D3    10
#define D4    11

#define PCLINE  0   // 0 = UART0 = PC-Schnitte
#define JBUS    1   // 1 = UART1 = J-BUS
#define JBUSMSG 50  // Gre der union jbusdataset, (4 + 4 + 21 + 21) Bytes

#define SPEED       0
#define RPM         1
#define AUSSENTEMP  2
#define COOLANT     3
#define IGNITION    4
#define COMPFAN     5
#define FREE        6

#define PID_SPEED        84 //  84 = 0x54
#define PID_RPM         190 // 190 = 0xBE
#define PID_AUSSENTEMP  171 // 171 = 0xAB
#define PID_COOLANT     110 // 110 = 0x6E
#define PID_IGNITION     43 //  43 = 0x2B
#define PID_COMPFAN     200 // 200 = 0xC8 = Reserved PID
#define PID_FREE        201 // 201 = 0xC9 = Reserved PID

// Dauer der temp. Abschaltung (in ms) fr serielle Schnitte nach Empfang eienr falschen Adresse
#define UART0RECV_DISABLETIME		5

// frei definierter Fehlercode
#define EBADCRC         50      /* CRC check failed */

// aus /usr/include/asm-generic/errno-base.h
#define E2BIG            7      /* Argument list too long */
#define EAGAIN          11      /* Try again */
#define EINVAL          22      /* Invalid argument */

// aus /usr/include/asm-generic/errno.h
#define ENODATA         61      /* No data available */
#define ETIME           62      /* Timer expired */
#define EPROTO          71      /* Protocol error */
#define EOVERFLOW       75      /* Value too large for defined data type */
#define EILSEQ          84      /* Illegal byte sequence */
#define EMSGSIZE        90      /* Message too long */
#define ENOBUFS         105     /* No buffer space available */
#define ETIMEDOUT       110     /* Connection timed out */
#define EWOULDBLOCK     EAGAIN  /* Operation would block */

#define USARTERR 3
#define USARTLEN 5
#define TIMEOUT 9
#define MAX 256
#define MIN 0

#define USART_RX_BUFFER_MASK  127
//#define EEPROM_SECTION  __attribute__ ((section (".eeprom")))
//#define ERRORLED (PORTD = 0x7F) // Port PD7 an

#define UART0_BAUD_RATE     57600
#define UART1_BAUD_RATE     9600

#define EnableSerial0Receiver       UCSR0B |= (1<<RXEN0);
#define DisableSerial0Receiver      UCSR0B &= ~(1<<RXEN0);
#define EnableSerial1Receiver       UCSR1B |= (1<<RXEN1);
#define DisableSerial1Receiver      UCSR1B &= ~(1<<RXEN1);

#define EnableSerial0Transmitter    UCSR0B |= (1<<TXEN0);
#define DisableSerial0Transmitter   UCSR0B &= ~(1<<TXEN0);
#define EnableSerial1Transmitter    UCSR1B |= (1<<TXEN1);
#define DisableSerial1Transmitter   UCSR1B &= ~(1<<TXEN1);

#define StartTimer1			TCCR1B |= (1<<WGM12) | (1<<CS12) | (1<<CS10);  // Prescaler = 1024, CTC, Taktgen. aktiv
#define StopTimer1			TCCR1B &= ~((1<<WGM12) | (1<<CS12) | (1<<CS11) | (1<<CS10));
#define EnableTimer1A		TIMSK |= (1<<OCIE1A);  // Timer 1A CTC Interrupt aktivieren
#define DisableTimer1A	TIMSK &= ~(1<<OCIE1A); // Timer 1A CTC Interrupt abschalten
#define EnableTimer1B		TIMSK |= (1<<OCIE1B);  // Timer 1B CTC Interrupt aktivieren
#define DisableTimer1B	TIMSK &= ~(1<<OCIE1B); // Timer 1B CTC Interrupt abschalten

#define StartTimer2			TCCR2 = (1<<WGM21) | (1<<CS22) | (1<<CS21) | (1<<CS20); // PRSC = 1024, CTC, Taktgen. aktiv
#define StopTimer2      TCCR2 &= ~((1<<CS22) | (1<<CS21) | (1<<CS20)); // Taktgenerator gestoppt
#define EnableTimer2		TIMSK |= (1<<OCIE2);  // Timer 2 Interrupt einschalten
#define DisableTimer2   TIMSK &= ~(1<<OCIE2);  // Timer 2 Interrupt ausschalten

#define StartTimer3			TCCR3B |= (1<<WGM32) | (1<<CS30);  // Prescaler = 1, CTC Modus, Taktgenerator aktiv
#define StopTimer3   		TCCR3B &= ~((1<<WGM32) | (1<<CS32) | (1<<CS31) | (1<<CS30)); // PRSC = 1, CTC, Taktgenerator gestoppt
#define EnableTimer3A		ETIMSK |= (1<<OCIE3A);  // Timer 1A CTC Interrupt aktivieren
#define DisableTimer3A	ETIMSK &= ~(1<<OCIE3A); // Timer 1A CTC Interrupt abschalten
#define EnableTimer3B		ETIMSK |= (1<<OCIE3B);  // Timer 1A CTC Interrupt aktivieren
#define DisableTimer3B	ETIMSK &= ~(1<<OCIE3B); // Timer 1A CTC Interrupt abschalten


#define EnableExtInt0		GIFR |= (1<<INTF0); GICR |=  (1<<INT0); // ext. Interrupt 0 aktivieren
#define DisableExtInt0  GICR &= ~(1<<INT0); // ext. Interrupt 0 sperren
#define ClrInt0Flag     GIFR |= (1<<INTF0);
//#define EnableExtInt0		MCUCR |= (1<<ISC01) // externen Interrupt 0 einschalten
//#define DisableExtInt0	MCUCR &= ~(1<<ISC01) // externen Interrupt 0 abschalten

// Uart1Stop und Uart1Start,
// Uart1Start = Datentransfer wird gestartet, Timer 1 & 3 anhalten, ext. Int. 0 abklemmen
// Uart1Stop = Datentransfer beendet, Timer weiterlaufen lassen, ext. Int. 0 einschalten
// 3 Befehle hintereinander
// Uart1Stop: Timer 1 CTC-Modus einstellen, Prescaler = 1024; Timer 3 CTC-Modus, Prescaler = 1; ext. Int. 0 aktivieren
// Uart1Start: Timer 1 & 3 Taktgenerator abgeklemmt, ext. Int. 0 deaktivieren
#define Uart1Stop	 TCCR1B |=  (1<<WGM12) | (1<<CS12) | (1<<CS10); TCCR3B |= (1<<WGM32) | (1<<CS30); GICR |= (1<<INT0);
#define Uart1Start TCCR1B &= ~((1<<WGM12) | (1<<CS12) | (1<<CS11) | (1<<CS10)); TCCR3B &= ~((1<<WGM32) | (1<<CS32) | (1<<CS31) | (1<<CS30)); GICR &= ~(1<<INT0);

#define TRUE		1
#define FALSE		0

//#define ClrTimeout                TIMSK2 &= ~(1<<TOIE2);
#define RxdLedOn      PORTB |= 0x01;
#define RxdLedOff     PORTB &= ~0x01;
#define TxdLedOn      PORTB |= 0x02;
#define TxdLedOff     PORTB &= ~0x02;

#define DebugLed1On		PORTA |= 0x04;
#define DebugLed1Off	PORTA &= ~0x04;
#define DebugLed2On		PORTA |= 0x08;
#define DebugLed2Off	PORTA &= ~0x08;
#define DebugLed3On		PORTA |= 0x10;
#define DebugLed3Off	PORTA &= ~0x10;
#define DebugLed4On		PORTA |= 0x20;
#define DebugLed4Off	PORTA &= ~0x20;
#define DebugLed5On		PORTA |= 0x40;
#define DebugLed5Off	PORTA &= ~0x40;
#define DebugLed6On		PORTA |= 0x80;
#define DebugLed6Off	PORTA &= ~0x80;
#define DebugLedsOff	PORTA &= ~(0x80 | 0x40 | 0x20 | 0x10 | 0x08 | 0x04);

