/*** FESTLEGUNGEN  *****************************************************************

1 Datenformat J-Bus Message
  PRIO    Prio-Level = 10 x t_Bit + (2 x t_Bit) x PRIO = 12 t_Bit x Prio = 1,25ms x PRIO
  XMIT    XMIT = 1 = mu gefunkt werden
  CNT     CNT wird im Prio-Timer hochgezhlt
  LEN     Lnge Datensatz data1...n
  data1
  data2
  data...
  data21

2 Timer
  Timer 1A, 16 Bit, Funktion: CCNT runterzhlen, Timeout-Handling, Aufl. 100 ms
  Timer 1B, 16 Bit,
  Timer 2, 8 Bit, Funktion: akt. Warteschleife v. 0,5 ms, danach Timer z. kuzzeitigen stoppen (10 Bitzeiten) von Timer 1A
  Timer 3A, 16 Bit, Funktion: PCNT runterzhlen, Aufl. max. 100 s
  //Timer 3B, 16 Bit, Funktion: Serielle Schnitte 0 (PC-Simulation) vorbergehend deaktivieren

3 UART
  UART0: PC <-> J-Bus Simu
  UART1: J-Bus Simu <-> J-Bus

  Steuerleitungen:
  (CTS, Clear To Send, Datenbertragungseinrichtung kann Daten bertragen, Pin 8 an 9pol. Sub-D)
        bertragungsgert (z. B. Modem) zeigt Empfangsbereitschaft an (Sendeerlaubnis)

  RTS, Request To Send, Datenendeinrichtung kann Daten bertragen, Pin 7 an 9pol. Sub-D
    0 = signalisiert der Gegenstelle (PC), dass sie Daten senden kann
    1 = Sendepause
        Endgert (z. B. PC) zeigt an, dass die Gegenstelle senden soll (Sendeanforderung)

4 Fuses
  HiFuse = 0xD9
  LoFuse = 0xCC

  > avrdude -p m162 -c stk500v2 -P /dev/ttyS0 -U hfuse:w:HiFuse:r
  > avrdude -p m162 -c stk500v2 -P /dev/ttyS0 -U lfuse:w:LoFuse:r
*******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include <util/crc16.h>
#include "uart.h"
#include "main.h"
#include "defs.h"
#include "crc16.h"

static char sw[20] = {'S','A','E','-','J','1','5','8','7',' ','V','e','r','.',' ','0','.','1','\n','\r'};
static volatile unsigned char sreg;  // fr SREG-Werte, Interrupt enable sei()-Ersatz
static volatile unsigned char tcnt2; // Reload-Wert f. TCNT2 (Int. Timer 2 Overflow)
static volatile unsigned int waittime;
static volatile unsigned char bytercv0; // =1, wenn Byte via UART0 angekommen, wird nur in ISR gesetzt
static unsigned char bytecount;
static volatile unsigned char next0; // nchstes Zeichen kann gesendet werden (= 1), UART0
static volatile unsigned char next1; // nchstes Zeichen kann gesendet werden (= 1), UART1
static volatile unsigned char data1, stat1; // in SIGNAL(UART1_RECEIVE_INTERRUPT) verwendet
//static volatile unsigned char uart1rcv[UART1MAXRCVLEN]; // Datenempfangspuffer v. UART1, 25 Bytes
static volatile unsigned char uart1rcv_cnt; // Zhlvar. f. Datenempfangspuffer v. UART1
static volatile unsigned char uart0firstbyte; // Flag, ob Byte von UART0 als erstes Byte identifiziert wird
static volatile unsigned char uart0rcvcmpl; // Flag, zeigt an, ob alle Bytes angekommen sind
static volatile unsigned char uart0incnt;  // Zhler fr eingehende Zeichen
static volatile unsigned char flag_jbusrcv; // zeigt an, ob gegenwrtig ein Datenbyte vom J-Bus eingetroffen ist

static volatile unsigned char uart0offcnt; // Zhlvar. f. Timer 3B Int., 0 = UART0 wieder empfangsbereit (temp. Empf.sperre)
static volatile unsigned char flag_uart0datain; // Flag, zeigt an, da ber UART0 (PC) falsche Adresse angekommen ist
static volatile unsigned char timer2cnt; // Zhlvariable f. Timer 2
static volatile unsigned char extint0;  // Flag, zeigt an, dass ext. Interrupt 0 ausgeloest wurde
static volatile unsigned char sendwait; // Flag fr Timer2, Buszugriffssteuerung

static volatile unsigned char pcbuf[MAXMSGLEN]; // MAXMSGLEN defined in defs.h, 30 Zeichen max.

static volatile unsigned char j, port;
static volatile unsigned int uartinput0, uartinput1;
static /*unsigned*/ volatile char jbus[21]; // Speicherplatz fr eingehende Daten von ECU
static /*unsigned*/ char jbusrcv_multisection[4][21];
static unsigned int rcvcmpl; // Framestatus speichern, max 16 Bit

unsigned int crc16_update(unsigned int, volatile unsigned char);
void scheduler(void);
void outstr0(unsigned char);
void outstr1(unsigned char);
void ClrTimeout(void); // berarbeiten/wegwerfen
void SetTimeout(unsigned char maxtime); // soll nach  N x 100s Timeout melden (Rcv. PC-Daten, Fehlerfall)
void UpdateMsg(void);
void NewError(void);
void NewFree(unsigned char i);
void DeleteError(void);
void CommandError(void);
void HwCheck(void);
void UART0Puts(/*unsigned volatile*/ char *str, unsigned char cnt, unsigned char crc);
unsigned char UART1Puts(unsigned volatile char *str, unsigned char cnt);

void main(void) __attribute__((noreturn));

ISR(SIG_INTERRUPT0)
{
  /*
  Diese ISR darf nur aufgerufen werden, wenn keine Datenbertragung mit ECU aktiv ist. In der ISR zur
	Datenbertragung zur ECU mu diese ISR abgeschaltet werden und nach Beendigung wieder aktiviert werden.

	Verhalten/Aktionen bei einer (ersten) Flanke an RxD:
	Es darf keine Datenbertragung aktiv sein
	- Interrupt SIG_INTERRUPT0 sperren
	- Timer 1A (CCNT) Stop, CCNT-Zhler nicht resetten
	- Timer 2 Start (Auflsung ca. eine Bytezeit), Timer 3A = PRSC=1, t_INT = 1/7372800 Sek, Init.-Val. = 7860
	  Timer 2 interruptierbares, aktives Warten (Interrupt zum Datenempfang v. PC)
	nach Timerablauf:
	- alle Buszugrifszeitenzhler Priocounter PCNT resetten
  - Timer 1A weiterlaufen lassen
	- Interrupt SIG_INTERRUPT0 freigeben

	- Datenbertragung aktiv
	Fehler von ECU
	*/

	sreg = SREG;
	cli();

	// diesen Interrupt sperren, nur einmaliger Aufruf mglich, Reaktivierung in Timer2-ISR
	DisableExtInt0; // GICR &= ~(1<<INT0) ext. Interrupt 0 sperren

	// alle Timer zur Buszugriffssteuerung (Timer 1 & 3) vorbergehend deaktivieren
  StopTimer1;
	StopTimer3;

  // Timer 2 deaktivieren fr den Fall, da nach einem Datenpaket von ECU ein weiteres Datenpaket von ECU kommt
	// und die Simulation auch funken will, aber etwas spter
	StopTimer2;
	DisableTimer2;
	TCNT2 = 0;

	SREG = sreg;
}

/*
ISR (__vector_default)
{
  PORTB = ~PORTB;
}
*/

// Timeouthandling, wird nur 1x aufgerufen (Selbstabschaltung)
ISR(SIG_OUTPUT_COMPARE2)
{
  sreg = SREG;
  cli();

	if(sendwait)
	{
		sendwait = 0;
		StartTimer1;
		StartTimer3;
		StopTimer2;    // Timer 2 Clock Source abklemmen
		DisableTimer2; // Timer 2 Interrupt aus (10 t_Bit sind um) u. Timer 1 & 3 aktivieren (Buszugriff)
  }

	if(flag_uart0datain) // Flag zeigt an, da Timer 2 durch UART0 aktiviert wurde (falsche Adresse, Data Overflow)
	{
	  timer2cnt++;
		if(timer2cnt == 40) // 40 x 1.25ms = 50 ms, Dauer der UART0 Deaktivierung
		{
		  timer2cnt = 0;
			EnableSerial0Receiver; // ISR(SIG_USART0_RECV) wieder aktivieren
			flag_uart0datain = 0;
      StopTimer2;    //
      DisableTimer2; // TIMSK &= ~(1<<OCIE2);  Timer 2 deaktivieren (10 t_Bit sind um) u. Timer 1 & 3
		}
	}

  SREG = sreg;
}


ISR(SIG_USART0_RECV)
{
	sreg = SREG;
  cli();

	pcbuf[uart0incnt] = UDR0;
	//if(uart0incnt == 0)
	if(pcbuf[0] == OWNADR)
	{
		uart0incnt++;

		if(uart0incnt == pcbuf[1]+1)
		{
			//DisableSerial0Receiver;
			//uart0incnt = 0;
			uart0rcvcmpl = 1;
		}
	}

	SREG = sreg;
}


//ISR(UART1_RECEIVE_INTERRUPT)
ISR(SIG_USART1_RECV)
/*************************************************************************
    Function: UART1 Receive Complete interrupt
    Purpose:  called when the UART1 has received a character
**************************************************************************/
{
	sreg = SREG;
  cli();

  // Reset Timer3b
  TCNT3 = 0;     // Timer/Counter Register auf 0;

  // alle Prio-Counter wieder auf ursprnglichen Wert zurcksetzen, weil
  // alle nochmal von vorne ihre Prio-Zeit absitzen mssen nach Datenempfang (Buszugriff)
  messages.speed[PCNT]      = messages.speed[PRIO];
  messages.rpm[PCNT]        = messages.rpm[PRIO];
  messages.aussentemp[PCNT] = messages.aussentemp[PRIO];
  messages.coolant[PCNT]    = messages.coolant[PRIO];
  messages.ignition[PCNT]   = messages.ignition[PRIO];
  messages.compfan[PCNT]    = messages.compfan[PRIO];
  messages.free[PCNT]       = messages.free[PRIO];

  // read UART status register and UART data register
  data1 = UDR1;     // UDR1 = UART1 DATA
  stat1 = UCSR1A;   // UCSR1A = UART1_STATUS

  if(stat1 == 0x20) // alles ok
  {
		flag.uart1rcv = 1;
    uart1rcv_cnt++;
    if(uart1rcv_cnt > UART1MAXRCVLEN)
      flag.uart1overflow = 1;
  }

	flag_jbusrcv = 1;  // Flag, zeigt an, ob Datenbyte vom J-Bus Daten vorhanden ist

	SREG = sreg;
}


void main(void)
{
  unsigned char ret, len, i, j, cnt, flag_jbusmsg_ok;
  unsigned char jbusmsgcmpl, jcnt, jmaxlen, jmulti, jmultixmit;

  wdt_disable();

  ret = 0;
  len = 0;

  jbusmsgcmpl = FALSE; // J-Bus Msg. noch nicht vollstndig
	flag_jbusmsg_ok = FALSE;
  jcnt = 0;
  jmulti = 0; // Nr. des Multiframes, Wertebereich 0...14 = 15 x 21 Bytes max.
  jmaxlen = 0;
  jmultixmit = 0;
  cnt = 0;

  InitPeripherie();

  //sei();
  //HwCheck(); // LEDs mal durchleuchten lassen
  //cli();

  // Timer Interrupts freigeben, TIMSK f. Timer 1A und 2, ETIMSK fr Timer 3A
	EnableTimer1A;
	EnableTimer3A;

  sei();

  UART0Puts(sw, 20, NOCRC);

  EnableSerial0Receiver;
  EnableSerial1Receiver;

  sreg = SREG;
	cli();
  InitVars(PLAIN);
  SREG = sreg;

  while(1)
  {
	  //*********************************************************************
		//**** PC Daten (Parametrisierung) Handhabung *************************
		//*********************************************************************
		if(uart0rcvcmpl) // alle Bytes angekommen
		{
      // Hier geht es los. Wenn der Datensatz vollstndig vom PC angekommen ist
      // wird der CRC-Check gemacht. In pcbuf[] sind die Daten richtig enthalten
		  ret = crc16(pcbuf, IN); // crc16(adr of beginning buffer, direction);
      // hier ist das Problem ENDE
      if(ret != 0) // CRC-error
			{
        outstr0(EBADCRC);
				outstr0(pcbuf[DLC]);
			}
      else // kein CRC16-Fehler
      {
        outstr0(ACK);
        switch(pcbuf[AKT]) // AKT = 2
        {
          case 'U': // update
            UpdateMsg();
            break;
          case 'N': // neuer Frame
            if(messages.free[6] > 0) // schon in use, Index 6 = MID
              NewFree(2);
            else
              NewFree(1);
            break;
          case 'D': // delete, 'D' = 0x44
            switch(pcbuf[PID_PC])
            {
              case PID_SPEED:
                messages.speed[PRIO] = 0; // Prio 0 = wird nicht mehr gesendet
                break;
              case PID_RPM:
                messages.rpm[PRIO] = 0; // PRIO 0 = wird nicht mehr gesendet
                break;
              case PID_AUSSENTEMP:
                messages.aussentemp[PRIO] = 0; // Prio 0 = wird nicht mehr gesendet
                break;
              case PID_COOLANT:
                messages.coolant[PRIO] = 0;
                break;
              case PID_IGNITION:
                messages.ignition[PRIO] = 0;
                break;
              case PID_COMPFAN:
                messages.compfan[PRIO] = 0;
                break;
              case PID_FREE:
                messages.free[PRIO] = 0;
                break;
              default:
                outstr0(EINVAL); // Invalid argument
            }
            break;
          case 'R': // Reset
            InitVars(PLAIN);
            break;

          case 'G':
            //outstr0(messages.speed[PRIO]);
            //outstr0(messages.rpm[PRIO]);
            //outstr0(messages.aussentemp[PRIO]);
            //outstr0(messages.coolant[PRIO]);
            //outstr0(TIMSK);
            //outstr0(ETIMSK);
            //outstr0(GIFR);
            //outstr0(GICR);
            //outstr0(TCCR1B);
            //outstr0(TCCR3B);
            if(rcvcmpl == 0) // noch keine Daten empfangen, also nix zum senden
            {
              outstr0(ENODATA); // No data available
              break;
            }
            // Datenkorrektur
            if(jbusrcv.indicatorlamp[1] > 0)
              jbusrcv.indicatorlamp[0] = 0x92;
            if(jbusrcv.aclamp[1] > 0)
              jbusrcv.aclamp[0] = 0x92;
            if(jbusrcv.diagnosticdata[1] > 0)
              jbusrcv.diagnosticdata[0] = 0x92;
            if(jbusrcv.c2[1] > 0)
              jbusrcv.c2[0] = 0x92;

            //UART0_puts(jbusrcv.all, JBUSMSG); // AC-Lamp[4], Indicator-Lamp[4], Diagnose[21], C2-Frame[21]
            UART0Puts(jbusrcv.all, JBUSMSG, WITHCRC); // AC-Lamp, Indicator-Lamp, Diagnose, C2-Frame, 54 Byte
            i = 0; // Schleifenzhler
            j = jbusrcv_multisection[0][4] >> 4; // j = Arraynummernzhler, Zeilennummer
            for(i = 0; i < 4; i++) // alle 4 Felder raus mit CRC nach jedem Feld
            {
              jbusrcv_multisection[i][0] = 0x92; // Korrektur, 1. Byte fehlt wg. Auswertung Datenempfang
              //UART0_puts(jbusrcv_multisection[i], 21); // einen kompl. Datensatz raus
              UART0Puts(jbusrcv_multisection[i], 21, WITHCRC);
            }
            //jbusrcv_multisection[i][0] = 0x92;
            //UART0_puts(jbusrcv_multisection[i], jbusrcv_multisection[i][2]+4);
            //UART0Puts(jbusrcv_multisection[i], jbusrcv_multisection[i][2]+4, WITHCRC);
						break;

          // BUG: Buszugriffszeiten einhalten!
          case 'P': // PassThrough , direkte Durchreiche
						//for(i = 0; i < 5; i++)  // 5 mal versuchen???
						//{
						  ret = UART1Puts(&pcbuf[ACK+1], pcbuf[DLC] - 3);
						  //if(ret == 0)
							//	i = 5;
						//}
						break;

          default: // ???
            outstr0(EILSEQ); //Error, Illegal byte sequence
        }  // switch(pcbuf[AKT])
      } // else (ret != 0) // CRC-error
			// Variablen resetten
			uart0rcvcmpl = 0; // letzter Datensatz (DS) wurde verarbeitet, erst dann wieder DS verarbeiten, wenn neuer DS vorh.
      //pcbuf[DLC] = 0;
      uart0firstbyte = 1; // nxtes Byte ist wieder das 1. im Frame
			uart0incnt = 0;     // Bytezhler resetten
		}

		/******************************************************************************************************
		*******************************************************************************************************
		******************************************************************************************************/

		scheduler(); // Subfkt. ausgelagert wegen bersichtlichkeit

    //********************************************************************
    // ******************  Datenempfang vom J-Bus, UART1  ****************
    //********************************************************************
    //********************************************************************
    //******** eingegangene Msg. vom SG absaugen, wenn vorhanden *********
    //********************************************************************
    //jcnt = 0;


// alte Datenanalyse (buggy)
    //if(flag.uart1rcv)
		if(flag_jbusrcv)
    {
			RxdLedOn;
      //flag.uart1rcv = 0;
			if(jcnt == 0)
			{
				if(data1 == 146) // 146 = Absenderadresse ECU
				{
					//while(flag.uart1rcv == 0) // solange warten, bis 2. Byte da ist; wird in ISR SIG_USART1_RCV auf 1 gesetzt
					//flag.uart1rcv = 0;
					jbus[jcnt] = data1;
					jcnt++; // jcnt = 1
					jbusmsgcmpl = FALSE;
				}
				else // Fehlerfall
					jbusmsgcmpl = TRUE;
			}

			flag_jbusrcv = 0;

      // Daten absaugen und einsortieren, jcnt = 1
      while(jbusmsgcmpl == FALSE) // FALSE = 0
      {
				if(flag_jbusrcv == 1) // es ist ein neues Byte vom J-Bus eingetroffen, einsortieren
				{
					jbus[jcnt] = data1;

					switch(jbus[1])  // 2. bis n. Byte checken, Speicherort bestimmen
					{
						case 44: // = PID 44
							jbusrcv.indicatorlamp[jcnt] = jbus[jcnt];
							jmaxlen = 4; //-1;
							break;
						case 50: // = PID 50
							jbusrcv.aclamp[jcnt] = jbus[jcnt];
							jmaxlen = 4;//-1;
							break;
						case 192: // Multisection-Frame(s), nur diese PID (192)
							//  0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20
							// 92 C0 11 C2 30 30 02 B3 01 05 B5 01 06 B5 01 09 B5 01 0B B5 CF
							// 92 C0 11 C2 31 ....
							// 92 C0 11 C2 32 ....
							// 92 C0 06 C2 33 ....
							//outstr0(data1);
							if(jcnt == 2) // Lngenangabe (Zeile), 3. Byte, max. 0x11 = 17
							{
								jmaxlen = jbus[2]+3+1; // jbus[2] = 0x11 = 17 -> 21 + 1, 1. Byte v. Folgeframe (0x92) mitnehmen
								outstr0(jmaxlen);
							}
							//if(jmaxlen > 0x11) // viel zu lang....
							//{
							//	jcnt = 0xFF; // Counter resetten,
							//	jbusmsgcmpl = TRUE; // Abbruchbedingung
							//	jmaxlen = 0;
							//}
							if(jcnt == 4) // Multiframe, 5. Byte
							{
								jmulti = jbus[4] & 0x0F;
								for(i = 0; i <= 4; i++)
									jbusrcv_multisection[jmulti][i] = jbus[i];
							}
							if(jcnt > 4)
								jbusrcv_multisection[jmulti][jcnt] = jbus[jcnt];
							break;
						case 194: // = 146 194 15 2 179 1 11 181 1 12 181 1 13 179 1 22 180 1 216 => 19 Byte
							jbusrcv.c2[jcnt] = jbus[jcnt];
							if(jcnt == 2)
								jmaxlen = jbus[2]+4;
							break;
						case 0xFE: // = PID 254
							jbusrcv.diagnosticdata[jcnt] = jbus[jcnt]; // Byte bernehmen
							if(jcnt == 3) // Lngenangabe
								jmaxlen = jbus[3]+4;
							break;
						//default: // Datenschrott
						//	jcnt = 0xFF; // Counter resetten,
						//	jbusmsgcmpl = TRUE; // Abbruchbedingung
						//	jmaxlen = 0;
					} // switch(jbus[1])

					jcnt++;

					if((jcnt > 3) && (jcnt >= jmaxlen)) // letztes/r Byte/Frame von (Multi-)Frame empfangen
					{
						jbusmsgcmpl = TRUE; // Abbruchbedingung
						jcnt = 0; // J-Bus Msg. vollstndig empfangen, Zhler jcnt resetten
						jmaxlen = 0;
						jmulti = 0;
						switch(jbus[1])
						{
							case 0x2C: rcvcmpl = INDICATORLAMP; break;
							case 0x32: rcvcmpl = ACLAMP; break;
							case 0xC0: rcvcmpl = MULTISECTION; break;
							case 0xC2: rcvcmpl = C2; break;
							case 0xFE: rcvcmpl = DIAGNOSTICDATA; break;
						}
					}
				  //outstr0(jcnt);
					/*
					else
					{
						jcnt++;

						if(jbusmsgcmpl == FALSE)
						{
							//while(flag.uart1rcv == 0) // solange warten, bis nxtes Byte angekommen ist

							//flag.uart1rcv = 0;
							//jcnt++; // Indexzhler einen weiter
						}

						PORTA = jcnt;
					}
					*/
					flag_jbusrcv = 0; // empfangene Date vom J-Bus ist einsortiert, Flag zurcksetzen
				} // if(flag_jbusrcv == 1) // if(jbusrcvbusy == 1)
      } // while(jbusmsgcmpl == FALSE)
			//StartTimer1;
			//StartTimer3;
			RxdLedOff;
			sendwait = 1; // Var. f. ISR TIMER2_COMPARE_MATCH, Busidle (1 Bytezeit) abwarten, dann erst rausfunken
			StartTimer2;  // PRSC = 1024, CTC, Taktgen. aktiv
			EnableTimer2; // Timer 2 (1.25 ms) starten, TIMSK |= (1<<OCIE2) = Timer 2 Interrupt enable
			EnableExtInt0; // ext. INT0 wieder aktivieren, wird in ISR SIG_INTERRUPT0 deaktiviert
			//StartTimer3;
    } // if(flag.uart1rcv) = Daten angekommen
		// alte (buggy) Datenanalyse ENDE

    jbusmsgcmpl = FALSE; // Wiederaufnahmebedingung
  } // while(1);

  //return 0;
}


// Durchlaufzeit 9,2s, wenn nix gesendet wird
void scheduler(void)
{
	unsigned char i;

  //********************************************************************
  //**************************  Scheduler  *****************************
  //********************************************************************
	if(messages.speed[PRIO] > 0) // Prio = 0 = wird nicht mehr gesendet
	{
		// funk mich bei nxter Gelegenheit
		if((messages.speed[CCNT] == 0) && (messages.speed[PCNT] == 0))
		{
			if(messages.speed[REP] > 0)
			{
				messages.speed[REP]--;
				if(messages.speed[REP] == 0)
					messages.speed[PRIO] = 0;
			}
			messages.speed[XLEN] |= 0x80; // Merker setzen, damit Msg. gefunkt werden darf
		}
	}

	if(messages.rpm[PRIO] > 0) // Prio = 0 = wird nicht mehr gesendet
	{
		if((messages.rpm[CCNT] == 0) && (messages.rpm[PCNT] == 0))
		{
			if(messages.rpm[REP] > 0)
			{
				messages.rpm[REP]--;
				if(messages.rpm[REP] == 0)
					messages.rpm[PRIO] = 0;
			}
			messages.rpm[XLEN] |= 0x80; // Merker setzen
		}
	}

	if(messages.aussentemp[PRIO] > 0) // Prio = 0 = wird nicht mehr gesendet
	{
		if((messages.aussentemp[CCNT] == 0) && (messages.aussentemp[PCNT] == 0))
		{
			if(messages.aussentemp[REP] > 0)
			{
				messages.aussentemp[REP]--;
				if(messages.aussentemp[REP] == 0)
					messages.aussentemp[PRIO] = 0;
			}
			messages.aussentemp[XLEN] |= 0x80; // Merker setzen
			//UART1Puts(&messages.aussentemp[MID], messages.aussentemp[XLEN]);
		}
	}

	if(messages.coolant[PRIO] > 0) // Prio = 0 = wird nicht mehr gesendet
	{
		if((messages.coolant[CCNT] == 0) && (messages.coolant[PCNT] == 0))
		{
			if(messages.coolant[REP] > 0)
			{
				messages.coolant[REP]--;
				if(messages.coolant[REP] == 0)
					messages.coolant[PRIO] = 0;
			}
			messages.coolant[XLEN] |= 0x80; // Merker setzen
			//UART1Puts(&messages.coolant[MID], messages.coolant[XLEN]);
		}
	}

	if(messages.ignition[PRIO] > 0) // Prio = 0 = wird nicht mehr gesendet
	{
		if((messages.ignition[CCNT] == 0) && (messages.ignition[PCNT] == 0))
		{
			if(messages.ignition[REP] > 0)
			{
				messages.ignition[REP]--;
				if(messages.ignition[REP] == 0)
					messages.ignition[PRIO] = 0;
			}
			messages.ignition[XLEN] |= 0x80; // Merker setzen
			//UART1Puts(&messages.ignition[MID], messages.ignition[XLEN]);
		}
	}

	if(messages.compfan[PRIO] > 0) // Prio = 0 = wird nicht mehr gesendet
	{
		if((messages.compfan[CCNT] == 0) && (messages.compfan[PCNT] == 0))
		{
			if(messages.compfan[REP] > 0)
			{
				messages.compfan[REP]--;
				if(messages.compfan[REP] == 0)
					messages.compfan[PRIO] = 0;
			}
			messages.compfan[XLEN] |= 0x80; // Merker setzen
			//UART1Puts(&messages.compfan[MID], messages.compfan[XLEN]);
		}
	}

	if(messages.free[PRIO] > 0) // Prio = 0 = wird nicht mehr gesendet
	{
		if((messages.free[CCNT] == 0) && (messages.free[PCNT] == 0))
		{
			if(messages.free[REP] > 0)
			{
				messages.free[REP]--;
				if(messages.free[REP] == 0)
					messages.free[PRIO] = 0;
			}
			messages.free[XLEN] |= 0x80; // Merker setzen
			//UART1Puts(&messages.free[MID], messages.free[XLEN]);
		}
	}

	bytecount = XLEN; // erstes auszuwertendes Element der Union, XLEN = 5
	//crashbit = 0;
	for(i = 0; i < 7; i++) // alle 7 Messages nach Sendefhigkeit abklappern
	{
		if(messages.allframes[bytecount] >= 0x80) // = 0x80 reif, > 0x80 berfllig
		{
			messages.allframes[bytecount] &= 0x7F; // XMIT-Bit for dem abfunken ausschalten
		  // sendefhige Message rausfunken, ExtInt0 vorher abschalten
			DisableExtInt0; // ExtInt0 abschalten, weil durch die Datenbertragung auf 1-Draht-Bus ein ExtInt0 ausgelst wird
			TxdLedOn;

			// funxoniert
			for(j = 1; j <= messages.allframes[bytecount]; j++)
			{
				outstr1(messages.allframes[bytecount+j]);
				uart1rcv_cnt--;  // wird in ISR UART1RCV hochgezhlt, hier wieder runterzhlen
				flag_jbusrcv = 0;
				flag.uart1rcv = 0; // wird auch in ISR UART1RCV gesetzt, wieder resetten
				//if(messages.allframes[bytecount+j] != data1) // falls Crash
				//{
					//PORTA = 0x01; // Alarm
				//}
			}
			// Ursprungszeitwerte wiederherstellen
			messages.allframes[bytecount-2] = messages.allframes[bytecount-4]; //bytecount-2 = PCNT, -4 = PRIO
			messages.allframes[bytecount-1] = messages.allframes[bytecount-3]; //bytecount-1 = CCNT, -3 = CYT
			//bytecount = bytecount + messages.allframes[bytecount] + XLEN + 1; //alt. Wert + XLEN + 6 Headerbytes
      // nachdem eine Message komplett rausgefunkt wurde, kurze Sendepause einlegen (BusIdle)
      StopTimer1;    // Timer 1 & 3 stoppen,
      StopTimer3;
      sendwait = 0xFF; // Flag fr ISR (SIG_OUTPUT_COMPARE2) setzen
      TCNT2 = 0;
      StartTimer2;    //  Timer 2 f. BusIdle starten
      EnableTimer2;
      while(sendwait) // nxte Msg erst dann abfunken, wenn BusIdle vorbei ist (unsaubere Methode!)
        ;
    }

    bytecount = bytecount + messages.allframes[bytecount] + XLEN + 1; //alt. Wert + XLEN + 6 Headerbytes

		TxdLedOff;
		EnableExtInt0;  // nachdem eine Msg. an ECU raus ist, ExtInt0 wieder einschalten
  }
		// neuen Offset berechnen, falls Crashbit nicht gesetzt ist
		/*
		if(crashbit == 0)
			bytecount = bytecount + messages.allframes[bytecount] + XLEN + 1; //alt. Wert + XLEN + 6 Headerbytes
		else // wenn Crashbit = 1, selben Frame nochmal senden, Schleifenzhler i um eins zurck
		{
			if(i > 0)
				i--;
			crashbit = 0;
		}
		*/
	//}
}



void outstr1(unsigned char c)
{
	//MCUCR  &= ~(1<<ISC01); // externen Interrupt 0 abschalten
  UDR1 = c;
  while(next1 == 0)
    ;
  next1 = 0;
	flag.uart1rcv = 0; // beim senden wird auch empfangen, also eigene Msg. nicht auswerten
	//MCUCR |= (1<<ISC01);	// externen Interrupt 0 wieder einschalten
}


void outstr0(unsigned char c)
{
  //UART0_putc(c);
  UDR0 = c;
  while(next0 == 0)
    ;
  next0 = 0;
}


void UART0Puts(/*unsigned volatile*/ char *str, unsigned char cnt, unsigned char calccrc)
{
  unsigned char i;
  union crcbytes crc;

  if(calccrc)
  {
    crc.all = 0xFFFF; // Startwert
    for(i = 0; i < cnt; i++)
    {
      //UART0_putc(str[i]); // Daten abfeuern
      UDR0 = str[i];
      crc.all = crc16_update(crc.all, str[i]);
      while(next0 == 0)
        ;
      next0 = 0;
    }
    // CRC16 anhngen und raus
    outstr0(crc.hi);
    outstr0(crc.lo);
  }
  else
    for(i = 0; i < cnt; i++)
      outstr0(str[i]);
}


// Einzelbytebertragung auf J-Bus mit Gegencheck
unsigned char UART1Puts(unsigned volatile char *str, unsigned char cnt)
{
  unsigned char i, uart1in;

  //while(sendwait)
  // ;
  UCSR1B &= ~(1<<RXCIE1); // RX Complete Interrupt deaktivieren
  DisableExtInt0; // ext. Int. 0 abschalten
	////StopTimer1; // Timer 1 anhalten (Taktgenerator abgeklemmt)
	////StopTimer3;

  for(i = 0; i < cnt; i++)
  {
    outstr1(str[i]); // Fkt. kehrt erst nach kompletten Datenversand zurck
    uart1in = UDR1;
    if(str[i] != uart1in) // Datenkollision?
      cnt = 0xFE; // Abbruchbedingung schaffen
  }

	flag.uart1rcv = 0; // beim senden wird auch empfangen, also eigene Msg. nicht auswerten

  UCSR1B |= (1<<RXCIE1);	// RX Complete Interrupt wieder aktivieren

  // nach dem Senden der Msg. ein BusIdle einfuegen, dazu Flag sendwait=1 setzen und Timer2 starten,
  // nach Ablauf von Timer2 (1.25ms) Sendefaehigkeit wieder herstellen (sendwait=0)
  // nachdem eine Message komplett rausgefunkt wurde, kurze Sendepause einlegen (BusIdle)
  StopTimer1;    // Timer 1 & 3 stoppen,
  StopTimer3;
  sendwait = 0xFF; // Flag fr ISR (SIG_OUTPUT_COMPARE2) setzen
  TCNT2 = 0;
  StartTimer2;    //  Timer 2 f. BusIdle starten
  EnableExtInt0;  // nachdem eine Msg. an ECU raus ist, ExtInt0 wieder einschalten
  EnableTimer2;
  while(sendwait) // nxte Msg erst dann abfunken, wenn BusIdle vorbei ist (unsaubere Methode!)
    ;

  if(cnt == 0xFE) // da war ne Kollision
    return 1;
  else
    return 0;
}


ISR(SIG_USART1_TRANS) // UART1 bertragung beendet (Byte)
{
  next1 = 1;
}


ISR(SIG_USART0_TRANS) // UART0 bertragung beendet (Byte)
{
  next0 = 1;
}


void InitVars(unsigned char plain)
{
  unsigned char i, j;

  next0 = 0;
  next1 = 0;

  if(plain == 0)
    flag.allbits = 0;

  uart1rcv_cnt = 0;
  rcvcmpl = 0;

	uart0firstbyte = 1;
	uart0rcvcmpl = 0;
  uart0incnt = 0;
	uart0offcnt = UART0RECV_DISABLETIME; // wird in ISR(SIG_OUTPUT_COMPARE3B) in 1 ms Schritten runtergezhlt
	flag_uart0datain = 0; // Flag, zeigt an, da ber UART0 (PC) falsche Adresse angekommen ist oder Overrun

	extint0 = 0; // Flag, zeigt an, dass ext. Interrupt ausgeloest wurde

	timer2cnt = 0; // zhlvariable f. Timer2 Interrupt (Compare Match)
	//timer2bcnt = 0;
	sendwait = 0;  // Var. zur Buszugrffssteuerung (Timer 2)

  // Reset Timer 1 & 3
  TCNT3 = 0;     // Timer/Counter Register auf 0;
  TCNT1 = 0;
	TCNT2 = 0;

	flag_jbusrcv = 0; // zeigt an, ob gegenwrtig Daten vom J-Bus eintreffen und verarbeitet werden
	//jbusrcvcnt = 0; // alle Bytes vom J-Bus zhlen, wird auf 0 gesetzt, wenn Frame vollstndig

  // Initialisierung mit Default-Werten
  messages.speed[REP] = 0;  // Repeat = 0 = immer funken
  messages.speed[PRIO] = 1; // Prio 1
  messages.speed[CYT] = 1;  // 1 = 100 ms
  messages.speed[PCNT] = messages.speed[PRIO];
  messages.speed[CCNT] = messages.speed[CYT];
  messages.speed[XLEN] = 4; // LEN inkl. CRC
  messages.speed[MID] = 144;  // MID (Sender) = 144 (=0x90) = Cruise Control
  messages.speed[PID] = 84; // PID = 84 = 0x54
  messages.speed[D1] = 0;   // data = 0 km/h default
  messages.speed[D2] = 28;  // CS = 28 = 0x1C

  messages.rpm[REP] = 0;
  messages.rpm[PRIO] = 1; // Prio 1
  messages.rpm[CYT] = 1;  // 1 = 100 ms
  messages.rpm[PCNT] = messages.rpm[PRIO];
  messages.rpm[CCNT] = messages.rpm[CYT];
  messages.rpm[XLEN] = 0x05; // LEN inkl. CRC
  messages.rpm[MID] = 128;  // MID (Sender) = 128 = 0x80 = Engine #1
  messages.rpm[PID] = 190; // PID = 190 = 0xBE
  messages.rpm[D1] = 0;    // data = 0
  messages.rpm[D2] = 0;    // data = 0 => 0 U/min
  messages.rpm[D3] = 194;// CS = 194 = 0xC2

  messages.aussentemp[REP] = 0;
  messages.aussentemp[PRIO] = 7;
  messages.aussentemp[CYT] = 100;  // 10 Sek.
  messages.aussentemp[PCNT] = messages.aussentemp[PRIO];
  messages.aussentemp[CCNT] = messages.aussentemp[CYT];
  messages.aussentemp[XLEN] = 0x05;
  messages.aussentemp[MID] = 140; // MID (Sender) = 0x8C = Instrument Cluster
  messages.aussentemp[PID] = 171; // 0xAB
  messages.aussentemp[D1] = 0; //54;  // 0x36, L-Byte, 54F = +12C
  messages.aussentemp[D2] = 214;// 214 = 0xD6  H-Byte, Wertebereich -8192F....+8191.75F (-4568.9C...4533.2C)
  messages.aussentemp[D3] = 243; // CS = 243 = 0xF3

  messages.coolant[REP] = 0;
  messages.coolant[PRIO] = 4;
  messages.coolant[CYT] = 10; // 1 Sek.
  messages.coolant[PCNT] = messages.coolant[PRIO];
  messages.coolant[CCNT] = messages.coolant[CYT];
  messages.coolant[XLEN] = 0x04;
  messages.coolant[MID] = 128;  // MID (Sender) = 128 = Engine #1
  messages.coolant[PID] = 110;  // PID 110 = 0x6E
  messages.coolant[D1] = 97; // Werteberich 0...255 Farenheit, C = (F-32)/1.8, 0x61 = 97F = 36.1C
  messages.coolant[D2] = 177; // CS = 177 = 0xB1

  messages.ignition[REP] = 0;
  messages.ignition[PRIO] = 0;
  messages.ignition[CYT] = 0;
  messages.ignition[PCNT] = 0;
  messages.ignition[CCNT] = 0;
  messages.ignition[XLEN] = 0x04;  // LEN inkl. CRC
  messages.ignition[MID] = 0;
  messages.ignition[PID] = 0;
  messages.ignition[D1] = 0;
  messages.ignition[D2] = 0;

  messages.compfan[REP] = 0;
  messages.compfan[PRIO] = 0;
  messages.compfan[CYT] = 0;
  messages.compfan[PCNT] = 0;
  messages.compfan[CCNT] = 0;
  messages.compfan[XLEN] = 0x05; // LEN inkl. CRC
  messages.compfan[MID] = 0;
  messages.compfan[PID] = 0;
  messages.compfan[D1] = 0;
  messages.compfan[D2] = 0;
  messages.compfan[D3] = 0;

  for(i = 0; i <= 27; i++)
    messages.free[i] = 0;

  bytercv0 = 0;

  for(i = 0; i < JBUSMSG; i++)
    jbusrcv.all[i] = 0;
  /*
  {
   jbusrcv_indicatorlamp[i] = 0;
    jbusrcv_aclamp[i] = 0;
  }
  */
  for(i = 0; i < 15; i++)
    for(j = 0; j < 21; j++)
      jbusrcv_multisection[i][j] = 0;
}


void InitPeripherie(void)
{
  /****************************************************
  Timer
  Timer 1A, 16 Bit, Funktion: CCNT runterzhlen, Timeout-Handling, Aufl. max. 100 ms
  Timer 1B, 16 Bit,
  Timer 2, 8 Bit    Funktion: akt. Warteschleife, Zeitbasis 0,5 ms, zeitliche Sperre/Abschaltung UART0 bei falscher Adresse
  Timer 3A, 16 Bit, Funktion: PCNT runterzhlen, Aufl. max. 100 s
  Timer 3B, 16 Bit, Timeouthandling Datenempfang UART0 und UART1, Aufl. 100s
  *****************************************************/

  // Initialisierung des 16-Bit Timers1A, Interrupt alle 100 ms bei 7.372800 MHz Quarz
  // PRSC = 1, 8, 64, 256, 1024
  // 100 ms = PRSC * OCR1A / f_OSC = 1024 * 720 / 7372800
  TCNT1  = 0;  // Timer Counter Register initialisieren
  OCR1A  = 719; // (PRSC x (OCR1x + 1)) / f_CPU = (1024 x (719 + 1)) / 7372800 MHz = 0,1 Sek
	//OCR1B  = 7;   // (1024 x (7 + 1)) / 7372800 MHz = 1,11 ms, Bytezeit@9600Bd = 1,042 ms
	StartTimer1;  // = TCCR1B = (1<<WGM12) | (1<<CS12) | (1<<CS10);  // Prescaler = 1024, CTC Modus

  // Timer 2 als CTC (normaler Timer braucht zuviel CPU-Zeit)
	OCR2 = 8; // (8+1 * PRSC) / f_OSC = (8+1 * 1024) / 7372800 = 1.25 ms (>1 Bytezeit (1.145833ms @9600,8N2)
	StartTimer2; // TCCR2  = (1<<WGM21) | (1<<CS22) | (1<<CS21) | (1<<CS20); CTC-Modus, Prescaler = 1024

  // Initialisierung des 16-Bit Timers3A, Interrupt alle 100 s bei 7.372800 MHz Quarz
  // PRSC = 1, 8, 16, 32, 64, 256, 1024
  // 100 s = PRSC * OCR3A / f_OSC = 1 * 737 / 7372800
  TCNT3  = 0;  // Timer Counter Register initialisieren
  OCR3A  = 737;      // Compare Match Register Timer 3A, soll 0,1 ms, ist ???? @7.3728MHz
	//OCR3B  = 7680;     // Compare Match Register Timer 3B, soll 1,0 ms, ist ???? @7.3728MHz
  //OCR3B  = 3686;   // Compare Match Register Timer 3B, soll 0,5 ms, ist ???? @7.3728MHz
  //OCR3B  = 22117;  // Compare Match Register Timer 3B, soll 3,0 ms, ist ???? @7.3728MHz
  //EnableTimer3; // = TCCR3B = (1<<WGM32) | (1<<CS30);  // Prescaler = 1, CTC Modus
	StartTimer3;

	// Richtung der Ports festlegen
  DDRA  = 0xFF;
  PORTA = 0x00; // Port A0...7 aus
  //DDRB = 0x03; // Port PB0, PB1 = Output
  DDRC = 0x07; // PC0, PC1, PC2 = Output
  //DDRC &=~ (1 << PC3); // Pin PC3 input
  //DDRD = 0xF0; // PD4...7 = Output
	DDRD = 0;
  //PORTB = 0xFF; // Port PB0, PB1 aus
  //PORTC = 0xFF; // Port C0...4 aus
  //PORTC |= (1 << PC3); // Pin PC3 pull-up enabled
  //PORTD = 0xFF; // Port D4...7 aus

  // UARTs initialisieren
  UART0_init(UART_BAUD_SELECT(UART0_BAUD_RATE, F_CPU)); // UART0_BAUD_RATE defined in main.h (57600)
  UART1_init(UART_BAUD_SELECT(UART1_BAUD_RATE, F_CPU)); // UART1_BAUD_RATE defined in main.h (9600)

  // ext. Interrupt 0 konfigurieren und aktivieren
  MCUCR |= (1<<ISC01); // Int. Auslsung bei fallender Flanke an Pin INT0
	EnableExtInt0; // GICR |= (1<<INT0);
}


// Timer 2, 0.5 ms Wartezeit???
void sleep8(unsigned char wait)
{
  // f_OSC = 7372800 Hz, PRSC = 32, TCNT2 = 141 = 115 Inkrementierungen
  // Wartezeit = (115 * 32) / 7372800 = 0.499 ms
  // PRSC = 2, 8, 32, 64, 128, 256, 1024
  tcnt2 = 141; // = 115 Inkrementierungen
  TCNT2 = tcnt2;
  waittime = wait; // waittime wird in ISR(SIG_OVERFLOW2) runtergezhlt
  TCCR2 = (1<<CS21) | (1<<CS20); // Timer 2, Prescaler = 32
  // Timer 0, 8-Bit initialisieren
  // 250 Inkrementierungen bis Interrupt -> 250 x 2us = 0.5 ms

  TIMSK |= (1<<TOIE2); // Timer Overflow Interrupt Enable 0 einschalten

  do
  {
    asm volatile("NOP"); // tu_was_gescheites();
  }while(waittime > 0);

  DisableTimer2; // TIMSK &= ~(1<<TOIE2);
  StopTimer2; //TCCR2 &= ~((1<<CS20) | (1<<CS21) | (1<<CS22)); // Timer Stop
}


void HwCheck(void)
{
  PORTA = 1;
  sleep8(250);
  PORTA = 2;
  sleep8(250);
  PORTA = 4;
  sleep8(250);
  PORTA = 8;
  sleep8(250);
  PORTA = 16;
  sleep8(250);
  PORTA = 32;
  sleep8(250);
  PORTA = 64;
  sleep8(250);
  PORTA = 128;
  sleep8(250);
	PORTA = 255;
	sleep8(250);
  PORTA = 0;
	PORTC = 1;
	sleep8(250);
	PORTC = 0;
}


void NewFree(unsigned char i)
{
  if(i == 2)
  {
    // Frame schon verwendet
  }

  if(i == 1)
  {
    // Daten reinkopieren
    messages.free[REP] = pcbuf[REP_PC];
    messages.free[PRIO] = pcbuf[PRIO_PC];
    messages.free[CYT] = pcbuf[CYT_PC];
    messages.free[PCNT] = 0;
    messages.free[CCNT] = 0;
    messages.free[MID] = pcbuf[MID_PC];
    messages.free[PID] = pcbuf[PID_PC];
    for(i = PID+1; i < 21; i++)
      messages.free[i] = pcbuf[i];
  }
}


// Restbussimulation beeinflussen
void UpdateMsg(void)
{
  switch(pcbuf[PID]) // pcbuf[7] == PID, die PID darf im messages.xxx nicht verndert werden!
  {
    case 84:    // 84 = 0x54 = Speed
      messages.speed[REP]  = pcbuf[REP_PC];
      messages.speed[PRIO] = pcbuf[PRIO_PC];
      messages.speed[CYT]  = pcbuf[CYT_PC];
      messages.speed[MID]  = pcbuf[MID_PC];
      messages.speed[D1]   = pcbuf[D1_PC];  // ein Datenbyte
      messages.speed[D2]   = pcbuf[D2_PC];  // CS
      break;

    case 110:  // 0x6E = 110 = Coolant
      messages.coolant[REP]  = pcbuf[REP_PC];
      messages.coolant[PRIO] = pcbuf[PRIO_PC];
      messages.coolant[CYT]  = pcbuf[CYT_PC];
      messages.coolant[MID]  = pcbuf[MID_PC];
      messages.coolant[D1]   = pcbuf[D1_PC];  // ein Datenbyte
      messages.coolant[D3]   = pcbuf[D2_PC];  // CS
      break;

    case 171: // 0xAB = 171 = Auentemp.
      messages.aussentemp[REP]  = pcbuf[REP_PC];
      messages.aussentemp[PRIO] = pcbuf[PRIO_PC];
      messages.aussentemp[CYT]  = pcbuf[CYT_PC];
      messages.aussentemp[MID]  = pcbuf[MID_PC];
      messages.aussentemp[D1]   = pcbuf[D1_PC];  // erstes Datenbyte
      messages.aussentemp[D2]   = pcbuf[D2_PC];  // zweites Datenbyte
      messages.aussentemp[D3]   = pcbuf[D3_PC];  // CS
      break;

    case 190:  // 0xBE = 190 = RPM
      messages.rpm[REP]  = pcbuf[REP_PC];
      messages.rpm[PRIO] = pcbuf[PRIO_PC];
      messages.rpm[CYT]  = pcbuf[CYT_PC];
      messages.rpm[MID]  = pcbuf[MID_PC];
      messages.rpm[D1]   = pcbuf[D1_PC];  // erstes Datenbyte
      messages.rpm[D2]   = pcbuf[D2_PC];  // zweites Datenbyte
      messages.rpm[D3]   = pcbuf[D3_PC];  // CS
      break;

    case 153: // CompFan
      messages.compfan[REP]  = pcbuf[REP_PC];
      messages.compfan[PRIO] = pcbuf[PRIO_PC];
      messages.compfan[CYT]  = pcbuf[CYT_PC];
      messages.compfan[MID]  = pcbuf[MID_PC];
      messages.compfan[D1]   = pcbuf[D1_PC];
      messages.compfan[D2]   = pcbuf[D2_PC];
      messages.compfan[D3]   = pcbuf[D3_PC];
      break;

    case 100: // = UpdateStatKl15()
      break;

    case 101: // free[2] = PID
      break;

    case 199: // ?
    case 200: // ?
      break;

    default:
      outstr0(EINVAL); // Invalid argument
      //UpdateError();
  }
}


// Timer 2 Overflow, 8-Bit Timer
ISR(SIG_OVERFLOW2)
{
  sreg = SREG;
  cli();
  waittime--;
  TCNT2 = tcnt2;
  SREG = sreg;;
}


// 16-Bit Timer Compare Match mit quasi Autoreload, wird alle 0.5 ms aufgerufen
// Wird zum runterzhlen der CYT-Variablen CCNT gebraucht. Erreicht CCNT 0, Nachricht
// bereit zum abfunken (Bus-Zugriff (PRIO) noch abwarten)
ISR(SIG_OUTPUT_COMPARE1A) // Durchlaufzeit 9,04s
{
  if(messages.speed[PRIO] > 0) // Prio 0 = wird nicht mehr gesendet
    if(messages.speed[CCNT] > 0)
      messages.speed[CCNT]--;

  if(messages.rpm[PRIO] > 0)
    if(messages.rpm[CCNT] > 0)
      messages.rpm[CCNT]--;

  if(messages.aussentemp[PRIO] != 0)
    if(messages.aussentemp[CCNT] > 0)
      messages.aussentemp[CCNT]--;

  if(messages.coolant[PRIO] != 0)
    if(messages.coolant[CCNT] > 0)
      messages.coolant[CCNT]--;

  if(messages.ignition[PRIO] != 0)
    if(messages.ignition[CCNT] > 0)
      messages.ignition[CCNT]--;

  if(messages.compfan[PRIO] != 0)
    if(messages.compfan[CCNT] > 0)
      messages.compfan[CCNT]--;

  if(messages.free[PRIO] != 0)
    if(messages.free[CCNT] > 0)
      messages.free[CCNT]--;

  /*
  if(flag.sleep)
    timer.wait++;
  */
}


// 16-Bit Timer Compare Match mit quasi Autoreload, wird alle 100 s aufgerufen
// Wird zum runterzhlen der PRIO-Variablen PCNT gebraucht (Bus-Zugriffszeit).
// Erreicht PCNT 0, Nachricht mu abgefunkt werden.
ISR(SIG_OUTPUT_COMPARE3A)
{
  if(messages.speed[CCNT] == 0) // Zykluszeit ist abgelaufen, Buszugriffszeit abwarten
    if(messages.speed[PCNT] > 0)
      messages.speed[PCNT]--;

  if(messages.rpm[CCNT] == 0) // Zykluszeit ist abgelaufen, Buszugriffszeit abwarten
    if(messages.rpm[PCNT] > 0)
      messages.rpm[PCNT]--;

  if(messages.aussentemp[CCNT] == 0) // Zykluszeit ist abgelaufen, Buszugriffszeit abwarten
    if(messages.aussentemp[PCNT] > 0)
      messages.aussentemp[PCNT]--;

  if(messages.coolant[CCNT] == 0) // Zykluszeit ist abgelaufen, Buszugriffszeit abwarten
    if(messages.coolant[PCNT] > 0)
      messages.coolant[PCNT]--;

  if(messages.ignition[CCNT] == 0) // Zykluszeit ist abgelaufen, Buszugriffszeit abwarten
    if(messages.ignition[PCNT] > 0)
      messages.ignition[PCNT]--;

  if(messages.compfan[CCNT] == 0) // Zykluszeit ist abgelaufen, Buszugriffszeit abwarten
    if(messages.compfan[PCNT] > 0)
      messages.compfan[PCNT]--;

  if(messages.free[CCNT] == 0) // Zykluszeit ist abgelaufen, Buszugriffszeit abwarten
    if(messages.free[PCNT] > 0)
      messages.free[PCNT]--;
}

/*
void _RegDump_(void)
{
  char tempbuf[10];

  //UART0Puts("MCUCR  ", 7, 0); itoa(MCUCR,  tempbuf, 16); UART0Puts(tempbuf, 2, 0); UART0Puts(" ", 1, 0);
  UART0Puts("GICR   ", 7, 0); itoa(GICR,   tempbuf, 16); UART0Puts(tempbuf, 2, 0); UART0Puts(" ", 1, 0);
  //UART0Puts("TIMSK  ", 7, 0); itoa(TIMSK,  tempbuf, 16); UART0Puts(tempbuf, 2, 0); UART0Puts(" ", 1, 0);
  //UART0Puts("ETMSK  ", 7, 0); itoa(ETIMSK, tempbuf, 16); UART0Puts(tempbuf, 2, 0); UART0Puts(" ", 1, 0);
  UART0Puts("TCCR1B ", 7, 0); itoa(TCCR1B, tempbuf, 16); UART0Puts(tempbuf, 2, 0); UART0Puts(" ", 1, 0);
  //UART0Puts("TCCR2  ", 7, 0); itoa(TCCR2,  tempbuf, 16); UART0Puts(tempbuf, 2, 0); UART0Puts(" ", 1, 0);
  UART0Puts("TCCR3B ", 7, 0); itoa(TCCR3B, tempbuf, 16); UART0Puts(tempbuf, 2, 0); UART0Puts(" ", 1, 0);
  outstr0(GICR);  outstr0(TCCR1B); outstr0(0x20);
}
*/
